/**
 * Desenvolvido por Mdulo Security Solutions S.A.
 * Contato: Eduardo Nery (enery@modulo.com.br - 61.218.7500)
 * Site: http://www.modulo.com.br
 **/

#ifndef KGREET_NSS_H
#define KGREET_NSS_H

#include "kgreeterplugin.h"
#include <qobject.h>

class KLineEdit;
class KPasswordEdit;
class QHBoxLayout;
class QLabel;


class KNSSGreeter : public QObject, public KGreeterPlugin
{
  Q_OBJECT
public:
  KNSSGreeter(KGreeterPluginHandler *handler, QWidget *parent, QWidget *predecessor,
              const QString &fixedEntitiy, Function func, Context ctx );

  virtual ~KNSSGreeter();

  /**
   * Variations of the talker:
   * - Authenticate: authentication
   * - AuthChAuthTok: authentication and password change
   * - ChAuthTok: password change
   */
  /*enum Function { Authenticate, AuthChAuthTok, ChAuthTok };*/
  /**
   * Contexts the talker can be used in:
   * - Login: kdm login dialog
   * - Shutdown: kdm shutdown dialog
   * - Unlock: kdm unlock dialog (TODO)
   * - ChangeTok: kdm password change dialog (TODO)
   * - ExUnlock: kdesktop_lock unlock dialog
   * - ExChangeTok: kdepasswd password change dialog (TODO)
   *
   * The Ex* contexts exist within a running session; the talker must know
   * how to obtain the currently logged in user (+ domain/realm, etc.)
   * itself (i.e., fixedEntity will be null). The non-Ex variants will have
   * a fixedEntity passed in.
   */
  /*enum Context { Login, Shutdown, Unlock, ChangeTok,
     ExUnlock, ExChangeTok };*/

  /**
   * Provide the talker with a list of selectable users. This can be used
   * for autocompletion, etc.
   * Will be called only when not running.
   * @param users the users to load.
   */
  virtual void loadUsers( const QStringList &users );

  /**
   * Preload the talker with an (opaque to the greeter) entity.
   * Will be called only when not running.
   * @param entity the entity to preload the talker with. That
   *  will usually be something like "user" or "user@domain".
   * @param field the sub-widget (probably line edit) to put the cursor into
   */
  virtual void presetEntity( const QString &entity, int field );

  /**
   * Obtain the actually logged in entity.
   * Will be called only after succeeded() was called.
   */
  virtual QString getEntity() const;

  /**
   * "Push" a user into the talker. That can be a click into the user list
   * or successful authentication without the talker calling gplugSetUser.
   * Will be called only when running.
   * @param user the user to set. Note that this is a UNIX login, not a
   *  canonical entity
   */
  virtual void setUser( const QString &user );

  /**
   * En-/disable any widgets contained in the talker.
   * Will be called only when not running.
   * @param on the state to set
   */
  virtual void setEnabled( bool on );

  /**
   * Called when a message from the authentication backend arrives.
   * @param message the message received from the backend
   * @param error if true, @p message is an error message, otherwise it's
   *  an informational message
   * @return true means that the talker already handled the message, false
   *  that the greeter should display it in a message box
   *
   * FIXME: Filtering a message usually means that the backend issued a
   * prompt and obtains the authentication data itself. However, in that
   * state the backend is unresponsive, e.g., no shutdown is possible.
   * The frontend could send the backend a signal, but the "escape path"
   * within the backend is unclear (PAM won't like simply longjmp()ing
   * out of it).
   */
  virtual bool textMessage( const char *message, bool error );

  /**
   * Prompt the user for data. Reply by calling handler->gplugReturnText().
   * @param propmt the prompt to display. It may be null, in which case
   *  "Username"/"Password" should be shown and the replies should be tagged
   *  with the respective Is* flag.
   * @param echo if true, a normal input widget can be used, otherwise one that
   *  visually obscures the user's input.
   * @param nonBlocking if true, report whatever is already available,
   *  otherwise wait for user input.
   */
  virtual void textPrompt( const char *prompt, bool echo, bool nonBlocking );

  /**
   * Request binary authentication data from the talker. Reply by calling
   * handler->gplugReturnBinary().
   * @param prompt prompt in pam_client format
   * @param nonBlocking if true, report whatever is already available,
   *  otherwise wait for user input.
   * @return always true for now
   *
   * TODO:
   * @return if true, the prompt was handled by the talker, otherwise the
   *  handler has to use libpam_client to obtain the authentication data.
   *  In that state the talker still can abort the data fetch by
   *  gplugReturn()ing a null array. When the data was obtained, another
   *  binaryPrompt with a null prompt will be issued.
   */
  virtual bool binaryPrompt( const char *prompt, bool nonBlocking );

  /**
   * This can either
   *  - Start a processing cycle. Will be called only when not running.
   *  - Restart authTok cycle - will be called while running and implies
   *    revive(). PAM is a bit too clever, so we need this.
   * In any case the talker is running afterwards.
   */
  virtual void start();

  /**
   * Request to suspend the auth. Make sure that a second talker of any
   * type will be able to operate while this one is suspended (no busy
   * device nodes, etc.).
   * Will be called only if running within Login context. (Actually it
   * won't be called at all, but be prepared.)
   */
  virtual void suspend();

  /**
   * Request to resume the auth from the point it was suspended at.
   * Will be called only when suspended.
   */
  virtual void resume();

  /**
   * The "login" button was pressed in the greeter.
   * This might call gplugReturn* or gplugStart.
   * Will be called only when running.
   */
  virtual void next();

  /**
   * Abort auth cycle. Note that this should _not_ clear out already
   * entered auth tokens if they are still on the screen.
   * Will be called only when running and stops it.
   */
  virtual void abort();

  /**
   * Indicate successful end of the current phase.
   * This is more or less a request to disable editable widgets
   * responsible for the that phase.
   * There will be no further attempt to enter that phase until the
   * widget is destroyed.
   * Will be called only when running and stops it.
   */
  virtual void succeeded();

  /**
   * Indicate unsuccessful end of the current phase.
   * This is mostly a request to disable all editable widgets.
   * The widget will be treated as dead until revive() is called.
   * Will be called only when running and stops it.
   */
  virtual void failed();

  /**
   * Prepare retrying the previously failed phase.
   * This is mostly a request to re-enable all editable widgets failed()
   * disabled previously, clear the probably incorrect authentication tokens
   * and to set the input focus appropriately.
   * Will be called only after failed() (possibly with clear() in between).
   */
  virtual void revive();

  /**
   * Clear any edit widgets, particularily anything set by setUser.
   * Will be called only when not running.
   */
  virtual void clear();

  /**
   * Obtain the QLayoutItem containg the widget(s) to actually handle the
   * conversation. See QLayout and QWidgetItem for possible implementations.
   */
  QLayoutItem *getLayoutItem() const { return layoutItem; }
  /*
  protected:
      KGreeterPluginHandler *handler;
      QLayoutItem *layoutItem;
  */

private:
  enum { FieldLogin = 1, FieldKeyPass};
  enum { StateLogin, StatePassword };

  void setPrompt(const char *prompt, bool echo);

  QHBoxLayout   *m_box;

  QLabel        *m_label;
  KLineEdit     *m_edLogin;
  KPasswordEdit *m_edPKPassword;


  QString    m_fixedUser;
  QString    m_curUser;
  Function   m_func;
  Context    m_ctx;
  int        m_state;
  bool       m_aborted;
};

#endif
