/**
 * Desenvolvido por Mdulo Security Solutions S.A.
 * Contato: Eduardo Nery (enery@modulo.com.br - 61.218.7500)
 * Site: http://www.modulo.com.br
 **/

#include "certutil.h"
#include "log.h"

CERTCertDBHandle *g_certHandle   = NULL;

/*
 * NSS call us to get the pvtkey password
 */ 
char* get_nss_password(PK11SlotInfo *slot, PRBool retry, void *arg) {
   char* pass;
  _log_debug("get_nss_password!");
  if(retry) {
    _log_debug("ops! retrying password, return 0");
    return 0;
  }
	
  g_key_request_flag = 1;

  if(g_keypass) {
    _log_debug("we have a password. Returning: %s", g_keypass);
    pass = (char*) malloc(sizeof(char) * strlen(g_keypass)+1);
    strcpy(pass, g_keypass);
    return pass;
  }

  _log_debug("we don't have a password! return 0");
  return NULL;	
  /*return getpass("Private key password: ");*/
}

/*
 * I'm not sure about this, but we need it :)
 * Oh, glorious copy/paste...
 */ 
SECStatus secu_StdinToItem(SECItem *dst)
{
    unsigned char buf[1000];
    PRInt32 numBytes;
    PRBool notDone = PR_TRUE;

    dst->len = 0;
    dst->data = NULL;

    while (notDone) {
        numBytes = PR_Read(PR_STDIN, buf, sizeof(buf));

        if (numBytes < 0) {
            return SECFailure;
        }

        if (numBytes == 0)
            break;

        if (dst->data) {
            /* XXX if PORT_Realloc fails, the old buffer is leaked. */
            dst->data = (unsigned char*)PORT_Realloc(dst->data,
                                                     dst->len + numBytes);
        } else {
            dst->data = (unsigned char*)PORT_Alloc(numBytes);

       }
        if (!dst->data) {
            return SECFailure;
        }
        PORT_Memcpy(dst->data + dst->len, buf, numBytes);
        dst->len += numBytes;
    }

    return SECSuccess;
}
/*
 * ...that my CTRL+C / CTRL+V shall never cease to exist
 */ 
SECStatus SECU_FileToItem(SECItem *dst, PRFileDesc *src)
{
    PRFileInfo info;
    PRInt32 numBytes;
    PRStatus prStatus;

    if (src == PR_STDIN)
        return secu_StdinToItem(dst);

    prStatus = PR_GetOpenFileInfo(src, &info);

    if (prStatus != PR_SUCCESS) {
        PORT_SetError(SEC_ERROR_IO);
        return SECFailure;
    }

    /* XXX workaround for 3.1, not all utils zero dst before sending */
    dst->data = 0;
    if (!SECITEM_AllocItem(NULL, dst, info.size))
        goto loser;

    numBytes = PR_Read(src, dst->data, info.size);
    if (numBytes != info.size) {
        PORT_SetError(SEC_ERROR_IO);
        goto loser;
    }

    return SECSuccess;
  loser:
    SECITEM_FreeItem(dst, PR_FALSE);
    return SECFailure;
}

/*
 * Process raw certificate 
 */ 
SECStatus SECU_ReadDERFromFile(SECItem *der, PRFileDesc *inFile, PRBool ascii)
{
    SECStatus rv;

    /* First convert ascii to binary */
    SECItem filedata;
    char *asc, *body;

      /* Read in ascii data */
      rv = SECU_FileToItem(&filedata, inFile);
      /*rv = SECU_FileToItem(der, inFile);*/
      asc = (char *)filedata.data;
      if (!asc) {
          _log_debug("unable to read data from input file");
          return SECFailure;
      }

      /* check for headers and trailers and remove them */
      if ((body = strstr(asc, "-----BEGIN")) != NULL) {
          char *trailer = NULL;
          asc = body;
          body = PORT_Strchr(body, '\n');
          if (!body)
              body = PORT_Strchr(asc, '\r');
              /* maybe this is a MAC file */
          if (body)
              trailer = strstr(++body, "-----END");
          if (trailer != NULL) {
              *trailer = '\0';
          } else {
              _log_debug("input has header but no trailer");
              PORT_Free(filedata.data);
              return SECFailure;
          }
      } else {
          body = asc;
      }

      /* Convert to binary */
      rv = ATOB_ConvertAsciiToItem(der, body);
      if (rv) {
          _log_debug("error converting ascii to binary");
          PORT_Free(filedata.data);
          return SECFailure;
      }

      PORT_Free(filedata.data);

    return SECSuccess;
}


CERTCertificate* retrieve_cert(PK11SlotInfo *g_slot, CERTCertDBHandle *handle, char *name, char *trusts,
        PRFileDesc *inFile, PRBool ascii, PRBool emailcert, void *pwdata)
{
    CERTCertificate *cert = NULL;
    CERTCertificate* impcert;
    SECItem certDER;
    SECStatus rv;

    certDER.data = NULL;

      /* Read in the entire file specified with the -i argument */
      rv = SECU_ReadDERFromFile(&certDER, inFile, ascii);
      if (rv != SECSuccess) {
          _log_debug("unable to read input file");
          return NULL;
      }

      /* Read in an ASCII cert and return a CERTCertificate */
      cert = CERT_DecodeCertFromPackage((char *)certDER.data, certDER.len);
      /* cert = CERT_DecodeDERCertificate(&certDER, PR_TRUE, "xyz");*/

      if (!cert) {
          _log_debug("could not obtain certificate from file");
          return NULL;
      }

      /******************************************/

    impcert = CERT_FindCertByName (g_certHandle,  &(cert->derSubject));


    CERT_DestroyCertificate (cert);
    /*PORT_Free(trust); */
    PORT_Free(certDER.data);
    
		return impcert;
}


CERTCertificate* get_nss_cert(const char* certxtpath) {
  PRFileDesc *inFile = NULL;
  CERTCertificate* cert;

	_log_debug("Opening file: %s", certxtpath);
	
	inFile = PR_Open(certxtpath, PR_RDONLY, 0);
  if (!inFile) {
    _log_debug("unable to open file \"%s\"for reading", certxtpath);/* (%ld, %ld).\n",  PR_GetError(), PR_GetOSError());*/
    return NULL;
  }

  cert = retrieve_cert(g_slot, g_certHandle, /*char *name*/ "", /*char *trusts,*/ "",
        inFile, /*PRBool ascii*/ PR_TRUE, /*PRBool emailcert*/ PR_FALSE, /*void *pwdata*/ NULL);

  return cert;
}


int init_nss() {
  SECStatus rv;

  
  PK11_SetPasswordFunc(get_nss_password);

  PR_Init(PR_SYSTEM_THREAD, PR_PRIORITY_NORMAL, 1);

  rv = NSS_Initialize(g_db_dir,"", "", g_db_file, NSS_INIT_READONLY);

  if (rv != SECSuccess) {
    _log_debug("NSS init failed\n");
    return SECFailure;
  }

  g_certHandle = CERT_GetDefaultCertDB();

  g_slot = PK11_GetInternalKeySlot();

  if(!g_slot) {
    _log_debug("PK11_GetInternalKeySlot failed\n");
    return SECFailure;
  } else {
    return SECSuccess;
  }
}

/*
 * Try get the private key so we know if password is needed 
 */ 
void check_privkey_pass(CERTCertificate* cert) {
	SECKEYPrivateKey* pvtkey = NULL;
  pvtkey =  PK11_FindKeyByAnyCert(cert, NULL);
	
  if(pvtkey) {
    SECKEY_DestroyPrivateKey(pvtkey);
	}
	
	return SECSuccess;
}

/*
 * Verify key integrity
 */ 
int chanllenge_it(CERTCertificate* cert) {
  SECKEYPrivateKey* pvtkey = NULL;
  SECKEYPublicKey   *pubkey = NULL;
  int                modulus_len, i, outlen;

  char *raw;
  char *enc;
  char *dec;
  SECStatus          rv;
  int ret;

  _log_debug("challenging...");
  
 pubkey = CERT_ExtractPublicKey(cert);
 if(pubkey == NULL)
 {
    _log_debug("Couldn't extract public key from cert");
    return SECFailure;
 }

  modulus_len = SECKEY_PublicKeyStrength(pubkey);
	
  raw = (char*) malloc(sizeof(char)*modulus_len);
  enc = (char*) malloc(sizeof(char)*modulus_len);
  dec = (char*) malloc(sizeof(char)*modulus_len);
	
	if(!(raw && enc && dec)) {
		return SECFailure;
	}
	
  for(i = 0; i < modulus_len; i++) {
    enc[i] = 0;
    dec[i] = 0;
		
		/* create a litle string */
    if(i < 10) {
      raw[i] = 50 + i;
    } else {
      raw[i] = 0;
    }
  }
	
  rv = PK11_PubEncryptRaw(pubkey, enc, raw, modulus_len, NULL);
  if (rv != SECSuccess)
  {
    _log_debug("Encrypt with Public Key failed");
    return SECFailure;
  }
  
  _log_debug("data encrypted...");


  pvtkey =  PK11_FindKeyByAnyCert(cert, NULL);
  if(!pvtkey) {
    _log_debug("unable to get privatekey. PK11_FindKeyByAnyCert error");
    return SECFailure;
  } else {
    _log_debug("we have a private key!");
  }

  rv = PK11_PubDecryptRaw(pvtkey, dec, &outlen, modulus_len, enc, modulus_len);
  if (rv != SECSuccess)
  {
      _log_debug("Decrypt with Private Key failed");
      return SECFailure;
  } else {
     _log_debug("decrypted!");
  }
  
  SECKEY_DestroyPrivateKey(pvtkey);
  SECKEY_DestroyPublicKey(pubkey);
  
  _log_debug("Result of decryption, dec  = \"%s\"", dec);

  ret = 0;
  ret = strcmp(raw, dec);
  return ret;
}
